/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//   ChanSearch performs a recursive binary search through frame files of 
//   a specified type to find the change-points in the channel contents of
//   the frames.
//
//   The command line syntax is as follows
//    ChanSearch --start-gps <start> --end-gps <end> --loc-file <file>
//               --cache-dir <direc>

#include "types.hh"
#include "frame_source.hh"
#include "chan_config.hh"
#include "scandir.hh"
#include <string>
#include <iostream>
#include <vector>

sends::frame_source sends::Source_List;
sends::chan_list    sends::Channel_List;

using namespace std;
using namespace sends;

//======================================  Channel search main function.
inline void
syntax(void) {
    cerr << "The command line syntax is as follows: " << endl;
    cerr << "ChanSearch --start-gps <start> --end-gps <end> --loc-file <file>"
	 << endl;
    cerr << "           --frame-type <type> --cache-dir <direc>" << endl;
}

//======================================  Channel search main function.
int
main(int argc, const char* argv[]) {
    
    //----------------------------------  Get the arguments
    string filematch = "/data/archive";
    string frame_type = "H-R";
    string cacheDir;
    gps_type start = 0;
    gps_type stop  = chan_entry::source_entry::inf_gps;
    for (int i=1; i<argc; i++) {
	string argi = argv[i];
	if (argi == "--cache-dir" || argi == "-d") {
	    cacheDir = argv[++i];
	}
	else if (argi == "--end-gps" || argi == "-e") {
	    stop = strtoul(argv[++i], 0, 0);
	}
	else if (argi == "--frame-type" || argi == "-f") {
	    frame_type = argv[++i];
	}
	else if (argi == "--loc-file" || argi == "-l") {
	    Source_List.read(argv[++i]);
	}
	else if (argi == "--start-gps" || argi == "-s") {
	    start = strtoul(argv[++i], 0, 0);
	}
	else {
	    cerr << "Unrecognized argument: " << argi << endl;
	    syntax();
	    return 1;
	}
    }
    //---------------------------------- get a better end-point
    if (!start) {
	const loc_list*  ll = Source_List.find_first(frame_type);
	if (!ll) {
	    cout << "No valid frames for type: " << frame_type << endl;
	    return 2;
	}
	start = ll->front().start();
    }

    if (stop  == chan_entry::source_entry::inf_gps) {
	const loc_list*  ll = Source_List.find_last(frame_type);
	if (!ll) {
	    cout << "No valid frames for type: " << frame_type << endl;
	    return 2;
	}
	stop = ll->back().stop() - ll->back().duration();
    }

    //---------------------------------- get the end-point lists
    chan_config topconf;
    std::vector<chan_config> stack(32);
    stack.clear();
    topconf.getChanList(frame_type, start);
    stack.push_back(topconf);
    topconf.getChanList(frame_type, stop);
    if (stack.back() == topconf) {
	topconf.merge(stack.back());
	stack.pop_back();
    }

    //----------------------------------  Loop until the stack is empty
    while (!stack.empty()) {
	cout << "top: " << topconf.start() << "-" << topconf.stop() 
	     << " next: " << stack.back().start() << "-" << stack.back().stop()
	     << endl;

	//------------------------------  Get the mid-point, 
	gps_type mid = (stack.back().stop() + topconf.start()) / 2;
	if (mid >= topconf.start()) {

	    //--------------------------  Add gaps to the end of the stack
	    if (topconf.empty()) stack.back().merge(topconf);

	    //--------------------------  Write real segments.
	    else                 topconf.write(cacheDir);

	    //--------------------------  pop off the top entry.
	    topconf = stack.back();
	    stack.pop_back();
	    continue;
	}

	//------------------------------  Test for a match one side or the other
	chan_config test;
	test.getChanList(frame_type, mid);

	//------------------------------  Match top?
	if (test == topconf) {
	    topconf.merge(test);
	}

	//------------------------------  No, Match end of stack?
	else if (test == stack.back()) {
	    stack.back().merge(test);
	}

	//------------------------------  No, push it on the stack
	else {
	    stack.push_back(test);
	}
    }
    topconf.write(cacheDir);
}
